@@ -1,7 +1,8 @@
+import time, pyperclip
 from kivy import Config
 import os, darkdetect, threading
-
 from kivymd.uix.button import MDFlatButton
+from kivymd.uix.picker import MDDatePicker
 
 Config.set('graphics', 'resizable', 0)
 Config.set('input', 'mouse', 'mouse, multitouch_on_demand')
@@ -13,21 +14,27 @@ from kivy.lang import Builder
 from kivy.uix.settings import ContentPanel
 from kivymd.app import MDApp
 from screeninfo import get_monitors
-from kivy.properties import StringProperty
+from kivy.properties import *
 from kivymd.uix.snackbar import Snackbar
 from kivymd.toast import toast
+from kivymd.icon_definitions import md_icons
+from kivymd.uix.list import IconLeftWidget, TwoLineAvatarIconListItem, IconRightWidget
+from kivymd.uix.list import OneLineAvatarIconListItem
 
 
-def getSpec(name: str, filename: str = 'main.py', icon: str = '', path: str = os.getcwd()):
+
+def icons(f: str = ''):
+    print([x for x in list(md_icons.keys()) if f in x])
+
+
+def getSpec(name: str, icon: str = 'icon.ico', filename: str = 'main.py', path: str = os.getcwd()):
     path = path.replace("\\", r"\\")
 
     with open(filename.replace('.py', '.spec'), 'w') as f:
-        f.write(fr"""
-from kivy_deps import sdl2, glew
+        f.write(fr"""from kivy_deps import sdl2, glew
 
 # -*- mode: python ; coding: utf-8 -*-
 
-
 block_cipher = None
 
 a = Analysis(['{filename}'],
@@ -77,6 +84,16 @@ def thread(func):
     def inner(*args, **kwargs):
         threading.Thread(target=lambda: func(*args, **kwargs), daemon=True).start()
 
+    return inner
+
+
+def tic(func):
+    def inner(*args, **kwargs):
+        tic = time.perf_counter()
+        func(*args, **kwargs)
+        toc = time.perf_counter()
+
+        print(f'Duration of {func.__name__}: {round(toc - tic, 8)}')
 
     return inner
 
@@ -88,7 +105,7 @@ class Kivy4(MDApp):
                  screen_size=None, minimum=None, center: bool = True,
                  sun_icon: str = 'white-balance-sunny', moon_icon: str = 'weather-night',
                  main_color: str = 'Blue', icon: str = '', toolbar=False, app_data: bool = True,
-                 custom_classes: bool = True, disable_x: bool = False, pre_string: str = '', **kwargs):
+                 disable_x: bool = False, pre_string: str = '', **kwargs):
 
         super().__init__(**kwargs)
 
@@ -96,6 +113,7 @@ class Kivy4(MDApp):
             minimum = [0.1, 0.1]
 
         self.app_name = app_name
+        self.builder_string = ''
 
         if app_data:
             app_data_path = os.getenv('APPDATA') + '/' + app_name
@@ -106,9 +124,7 @@ class Kivy4(MDApp):
             self.sun_icon = sun_icon
             self.isDarkMode()
 
-        self.theme_cls.primary_palette = main_color
-        self.disable_x = disable_x
-        self.icon = icon
+        self.setProperties(main_color, disable_x, icon, toolbar, string, pre_string)
 
         screen = get_monitors()[0]
         self.width = screen.width
@@ -116,43 +132,40 @@ class Kivy4(MDApp):
 
         self.screen_positions(screen_size, minimum, center)
 
-        self.builder_string = pre_string
+        self.run()
 
-        if custom_classes:
-            self.builder_string += self.custom_classes()
+    def setProperties(self, main_color, disable_x, icon, toolbar, string, pre_string):
+
+        for x in pre_string.split('\n'):
+            if 'x,y=' in x.replace(' ', ''):
+                self.builder_string += '\n' + self.x_y(x)
+
+            else:
+                self.builder_string += '\n' + x
+
+        self.builder_string += self.custom_classes()
 
         if toolbar:
             self.builder_string += self.getToolbar(toolbar)
 
-            z = string.split('\n')
-            for x in z:
+            for x in string.split('\n'):
                 if 'x,y=' in x.replace(' ', ''):
                     self.builder_string += '\n            ' + self.x_y(x)
 
                 else:
                     self.builder_string += '\n            ' + x
 
-        self.run()
-
-    @staticmethod
-    def x_y(x_y):
-        tabs = ""
-        for u in x_y:
-            if u == " ":
-                tabs += " "
-            else:
-                break
-
-        x, y = eval(x_y.split("=")[1])
-        return f"{tabs}pos_hint: {{'center_x': {x}, 'center_y': {y}}}"
+        self.theme_cls.primary_palette = main_color
+        self.disable_x = disable_x
+        self.icon = icon
 
     def build(self):
         self.use_kivy_settings = False
         self.settings_cls = ContentPanel
         self.title = self.app_name
 
-        Window.bind(on_request_close=lambda x: self.on_request_close(self.disable_x))
-        Window.bind(on_drop_file=self.on_drop_file)
+        Window.bind(on_dropfile=lambda *args: self.on_drop_file(*args),
+                    on_request_close=lambda x: self.on_request_close(self.disable_x))
 
         self.Build()
         return Builder.load_string(self.builder_string)
@@ -160,8 +173,13 @@ class Kivy4(MDApp):
     def Build(self):
         pass
 
-    def screen_positions(self, screen_size, minimum, center):
+    def screen_positions(self, screen_size, minimum=None, center=True):
+
+        if minimum is None:
+            minimum = [0.1, 0.1]
+
         min_x, min_y = minimum
+
         if screen_size is None:
             x, y = 0.6, 0.6
 
@@ -204,7 +222,6 @@ class Kivy4(MDApp):
             return e
 
     def setFile(self, file, value, extension='.txt'):
-
         path_to_create = f'{self.appdata_path}/{file}{extension}'
 
         try:
@@ -216,7 +233,6 @@ class Kivy4(MDApp):
             return e
 
     def getFile(self, file, default=None, create_file_if_not_exist=False, extension='.txt'):
-
         path_of_file = f'{self.appdata_path}/{file}{extension}'
 
         try:
@@ -239,7 +255,6 @@ class Kivy4(MDApp):
     def isDarkMode(self, filename='dark mode.txt'):
         try:
             with open(self.appdata_path + '/' + filename, 'r') as f:
-
                 current_mode = f.read()
                 self.setDarkModeIcon(current_mode)
 
@@ -264,24 +279,9 @@ class Kivy4(MDApp):
         self.setFile(filename, value)
         self.setDarkModeIcon(value)
 
-    @staticmethod
-    def toast(text, duration=2.5):
-        toast(text=text, duration=duration)
-
-    @staticmethod
-    def snack(text, button_text=None, func=None):
-        snack = Snackbar(text=text)
-
-        if func and button_text:
-            snack.buttons = [MDFlatButton(text=f"[color=#1aaaba]{button_text}[/color]",
-                                          on_release=func, ), ]
-
-        snack.open()
-
     def reverseDarkMode(self, filename: str = 'dark mode.txt'):
         try:
             with open(self.appdata_path + '/' + filename, 'r') as f:
-
                 current_mode = f.read()
 
                 if current_mode == 'Dark':
@@ -312,10 +312,6 @@ class Kivy4(MDApp):
 
         self.theme_cls.theme_style = value
 
-    @staticmethod
-    def on_request_close(disable_x: bool = False):
-        return disable_x
-
     def getToolbar(self, properties: list):
         if properties == True:
             right_icons, left_icons, name = '[[app.dark_mode_icon, lambda x: app.reverseDarkMode()]]', '[]', self.app_name
@@ -343,16 +339,36 @@ Screen:
             id: screen_manager
 '''
 
+    @staticmethod
+    def toast(text, duration=2.5):
+        toast(text=text, duration=duration)
+
+    @staticmethod
+    def snack(text, button_text=None, func=None):
+        snack = Snackbar(text=text)
+
+        if func and button_text:
+            snack.buttons = [MDFlatButton(text=f"[color=#1aaaba]{button_text}[/color]", on_release=func)]
+
+        snack.open()
+
+    @staticmethod
+    def x_y(x_y):
+        x, y = eval(x_y.split("=")[1])
+        return f"{x_y.index('x') * ' '}pos_hint: {{'center_x': {x}, 'center_y': {y}}}"
+
     @staticmethod
     def custom_classes():
         return '''
 <Text@MDLabel>:
     halign: 'center'
+    font_name: 'arial'
 
 <Input@MDTextField>:
     mode: "rectangle"
     text: ""
     size_hint_x: 0.5
+    font_name: 'arial'
 
 <Check@MDCheckbox>:
     group: 'group'
@@ -361,9 +377,11 @@ Screen:
 
 <Btn@MDFillRoundFlatButton>:
     text: ""
+    font_name: 'arial'
 
 <BtnIcon@MDFillRoundFlatIconButton>:
     text: ""
+    font_name: 'arial'
 
 <Img@Image>:    
     allow_stretch: True
@@ -372,6 +390,21 @@ Screen:
     md_bg_color: app.theme_cls.primary_color
 '''
 
+    def on_drop_file(self, *args):
+        print(*args)
+
     @staticmethod
-    def on_drop_file(_, path, x, y):
-        print(path, x, y)
\ No newline at end of file
+    def on_request_close(disable_x: bool = False):
+        return disable_x
+
+    @thread
+    def write_to_clipboard(self, text: str):
+        pyperclip.copy(text)
+
+    def show_date_picker(self, on_save, mode='picker'):
+        date_dialog = MDDatePicker(mode=mode)
+        date_dialog.bind(on_save=on_save, on_cancel=self.on_cancel_picker)
+        date_dialog.open()
+
+    def on_cancel_picker(self, instance, value):
+        pass
\ No newline at end of file
